@@ -1,8 +1,8 @@
 /* Extended Module Player
- * Copyright (C) 1996-98 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See docs/COPYING
+ * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
  */
 
@@ -10,7 +10,9 @@
  * 16 bit mono output.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -23,13 +25,11 @@
 #include "driver.h"
 #include "mixer.h"
 
-extern char *mixer_buffer;
-
 static int audio_fd;
 
-static int init (struct drv_config *);
-static int setaudio (struct drv_config *);
-static void bufdump (void);
+static int init (struct xmp_control *);
+static int setaudio (struct xmp_control *);
+static void bufdump (int);
 static void shutdown (void);
 
 static void dummy () { }
@@ -46,35 +46,35 @@ static char *help[] = {
     NULL
 };
 
-
 struct xmp_drv_info drv_hpux = {
     "hpux",		/* driver ID */
-    "HP-UX audio device",	/* driver description */
+    "HP-UX PCM audio",	/* driver description */
     help,		/* help */
     init,		/* init */
     shutdown,		/* shutdown */
-    smix_numvoices,	/* numvoices */
-    smix_voicepos,	/* voicepos */
-    smix_echoback,	/* echoback */
-    smix_setpatch,	/* setpatch */
-    smix_setvol,	/* setvol */
-    smix_setnote,	/* setnote */
-    smix_setpan,	/* setpan */
-    smix_setbend,	/* setbend */
+    xmp_smix_numvoices,	/* numvoices */
+    dummy,		/* voicepos */
+    xmp_smix_echoback,	/* echoback */
+    dummy,		/* setpatch */
+    xmp_smix_setvol,	/* setvol */
+    dummy,		/* setnote */
+    xmp_smix_setpan,	/* setpan */
+    dummy,		/* setbend */
+    dummy,		/* seteffect */
     dummy,		/* starttimer */
-    dummy,		/* stoptimer */
+    dummy,		/* stctlimer */
     dummy,		/* reset */
     bufdump,		/* bufdump */
     dummy,		/* bufwipe */
     dummy,		/* clearmem */
     dummy,		/* sync */
-    smix_writepatch,	/* writepatch */
-    smix_getmsg,	/* getmsg */
+    xmp_smix_writepatch,/* writepatch */
+    xmp_smix_getmsg,	/* getmsg */
     NULL
 };
 
 
-static int setaudio (struct drv_config *cfg)
+static int setaudio (struct xmp_control *ctl)
 {
     int flags;
     int gain = 128;
@@ -83,7 +83,7 @@ static int setaudio (struct drv_config *cfg)
     struct audio_gains agains;
     struct audio_describe adescribe;
     char *token;
-    char **parm = cfg->parm;
+    char **parm = ctl->parm;
     int i;
 
     parm_init ();
@@ -103,87 +103,86 @@ static int setaudio (struct drv_config *cfg)
         port = AUDIO_OUT_SPEAKER;
     }
 
-    flags = fcntl (audio_fd, F_GETFL, 0);
-
-    if (flags < 0)
-	return 0;
+    if ((flags = fcntl (audio_fd, F_GETFL, 0)) < 0)
+	return XMP_ERR_DINIT;
 
     flags |= O_NDELAY;
-    fcntl (audio_fd, F_SETFL, flags);
-
-    if (flags < 0)
-	return 0;
+    if ((flags = fcntl (audio_fd, F_SETFL, flags)) < 0)
+	return XMP_ERR_DINIT;
 
     if (ioctl (audio_fd, AUDIO_SET_DATA_FORMAT, AUDIO_FORMAT_LINEAR16BIT) == -1)
-	return 0;
+	return XMP_ERR_DINIT;
 
-    if (ioctl (audio_fd, AUDIO_SET_CHANNELS, cfg->mode + 1) == -1) {
-	cfg->mode = !cfg->mode;
-        if (ioctl (audio_fd, AUDIO_SET_CHANNELS, cfg->mode + 1) == -1)
-	    return 0;
-	cfg->mode++;
+    if (ioctl (audio_fd, AUDIO_SET_CHANNELS,
+	ctl->outfmt & XMP_FMT_MONO ? 1 : 2) == -1) {
+	ctl->outfmt ^= XMP_FMT_MONO;
+
+        if (ioctl (audio_fd, AUDIO_SET_CHANNELS,
+	    ctl->outfmt & XMP_FMT_MONO ? 1 : 2) == -1)
+	    return XMP_ERR_DINIT;
     }
 
     ioctl (audio_fd, AUDIO_SET_OUTPUT, port);
 
-    for (i = 0; ioctl (audio_fd, AUDIO_SET_SAMPLE_RATE, cfg->rate) == -1; i++) {
-	if ((cfg->rate = srate[i]) == 0)
-	    return 0;
-    }
+    for (i = 0; ioctl (audio_fd, AUDIO_SET_SAMPLE_RATE, ctl->freq) == -1; i++)
+	if ((ctl->freq = srate[i]) == 0)
+	    return XMP_ERR_DINIT;
 
     if (ioctl (audio_fd, AUDIO_DESCRIBE, &adescribe) == -1)
-	return 0;
+	return XMP_ERR_DINIT;
 
     if (ioctl (audio_fd, AUDIO_GET_GAINS, &agains) == -1)
-	return 0;
+	return XMP_ERR_DINIT;
 
     agains.transmit_gain = adescribe.min_transmit_gain +
 	(adescribe.max_transmit_gain - adescribe.min_transmit_gain) *
 	gain / 256;
 		
     if (ioctl (audio_fd, AUDIO_SET_GAINS, &agains) == -1)
-	return 0;
+	return XMP_ERR_DINIT;
 
     ioctl(audio_fd, AUDIO_SET_TXBUFSIZE, bsize);
 
-    return 1;
+    return XMP_OK;
 }
 	
 
-static int init (struct drv_config *cfg)
+static int init (struct xmp_control *ctl)
 {
-    audio_fd = open ("/dev/audio", O_WRONLY);
+    if ((audio_fd = open ("/dev/audio", O_WRONLY)) == -1)
+	return XMP_ERR_DINIT;
 
-    if (audio_fd == -1)
-	return 0;
+    if (setaudio (ctl) != XMP_OK)
+	return XMP_ERR_DINIT;
 
-    if (cfg->nbuf < 2)
-        cfg->nbuf = 2;
-
-    cfg->fmt &= 0xffff;
-
-    if (!setaudio (cfg))
-	return 0;
-
-    smix_init (cfg);
-
-    return 1;
+    return xmp_smix_on (ctl);
 }
 
 
 /* Build and write one tick (one PAL frame or 1/50 s in standard vblank
  * timed mods) of audio data to the output device.
  */
-static void bufdump ()
+static void bufdump (int i)
 {
-    int i;
+    int j;
+    void *b;
 
-    for (i = smix_mixer (); i -= write (audio_fd, mixer_buffer, i); );
+    /* Doesn't work if EINTR -- reported by Ruda Moura <ruda@helllabs.org> */
+    /* for (; i -= write (audio_fd, xmp_smix_buffer (), i); ); */
+
+    b = xmp_smix_buffer ();
+    while (i) {
+	if ((j = write (audio_fd, b, i)) > 0) {
+	    i -= j;
+	    (char *)b += j;
+	} else
+	    break;
+    };
 }
 
 
 static void shutdown ()
 {
-    smix_close ();
+    xmp_smix_off ();
     close (audio_fd);
 }
