@@ -1,9 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-98 Claudio Matsuoka and Hipolito Carraro Jr
- * AWE32 support Copyright (C) 1996, 97 Takashi Iwai
+ * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
+ * AWE32 support Copyright (C) 1996, 1997 Takashi Iwai
  *
  * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See docs/COPYING
+ * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
  */
 
@@ -22,6 +22,10 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
 #if defined(HAVE_SYS_SOUNDCARD_H)
 #include <sys/soundcard.h>
 #elif defined(HAVE_MACHINE_SOUNDCARD_H)
@@ -52,14 +56,14 @@
 #define AWE_DEVICE
 #endif
 
+#define SEQ_NUM_VOICES	32			/* #?# **** FIXME **** */
+
 SEQ_DEFINEBUF(2048);
 
-extern int __mode_fm;
-
-static int __echo_msg;
 static int seqfd = -1;
+static int echo_msg;
 
-static void numvoices	(int);
+static int numvoices	(int);
 static void voicepos	(int, int);
 static void echoback	(int);
 static void setpatch	(int, int);
@@ -67,6 +71,7 @@ static void setvol	(int, int);
 static void setnote	(int, int);
 static void setpan	(int, int);
 static void setbend	(int, int);
+static void seteffect	(int, int, int);
 static void starttimer	(void);
 static void stoptimer	(void);
 static void resetvoices	(void);
@@ -74,23 +79,21 @@ static void bufdump	(void);
 static void bufwipe	(void);
 static void clearmem	(void);
 static void seq_sync	(double);
-static void writepatch	(struct patch_info *);
-static int init		(struct drv_config *);
+static int writepatch	(struct patch_info *);
+static int init		(struct xmp_control *);
 static int getmsg	(void);
 static void shutdown	(void);
 
 #define seqbuf_dump bufdump
 
-
 static char *help[] = {
-    "chorus=mode,level", "Set chorus mode and level in AWE cards",
+    "awechorus=mode", "Set chorus mode in AWE cards",
+    "awereverb=mode", "Set reverb mode in AWE cards",
     "dev=<device_name>", "Device to use (default /dev/sequencer)",
-    "opl2", "Use OPL2 FM synthesizer",
-    "reverb=mode,level", "Set reverb mode and level in AWE cards",
+    "opl2", "Use OPL2 (YM3812) FM synthesizer",
     NULL
 };
 
-
 struct xmp_drv_info drv_oss_seq = {
     "oss_seq",		/* driver ID */
     "OSS sequencer",	/* driver description */
@@ -105,6 +108,7 @@ struct xmp_drv_info drv_oss_seq = {
     setnote,		/* setnote */
     setpan,		/* setpan */
     setbend,		/* setbend */
+    seteffect,		/* seteffect */
     starttimer,		/* settimer */
     stoptimer,		/* stoptimer */
     resetvoices,	/* resetvoices */
@@ -117,39 +121,47 @@ struct xmp_drv_info drv_oss_seq = {
     NULL
 };
 
-
 static int dev;
 static struct synth_info si;
-static int chorusmode = 0, choruslevel = 0;
-static int reverbmode = 0, reverblevel = 0;
+static int chorusmode = 0;
+static int reverbmode = 0;
 static char *dev_sequencer = "/dev/sequencer";
 
 
-static void numvoices (int n)
+static int numvoices (int num)
 {
-    if (si.synth_subtype == SAMPLE_TYPE_GUS) {
-	if (n < 14)
-	    n = 14;
-	GUS_NUMVOICES (dev, n);
+    switch (si.synth_subtype) {
+    case SAMPLE_TYPE_GUS:
+	if (num < 14)
+	    num = 14;
+#ifdef AWE_DEVICE
+    case SAMPLE_TYPE_AWE32:
+#endif
+	if (num > SEQ_NUM_VOICES)
+	    return SEQ_NUM_VOICES;
+	GUS_NUMVOICES (dev, num);
+	break;
     }
+
+    return num;
 }
 
 
-static void voicepos (int ch, int i)
+static void voicepos (int ch, int pos)
 {
-    GUS_VOICE_POS (dev, ch, i);
+    GUS_VOICE_POS (dev, ch, pos);
 }
 
 
-static void echoback (int i)
+static void echoback (int msg)
 {
-    SEQ_ECHO_BACK (i);
+    SEQ_ECHO_BACK (msg);
 }
 
 
-static void setpatch (int ch, int n)
+static void setpatch (int ch, int smp)
 {
-    SEQ_SET_PATCH (dev, ch, n);
+    SEQ_SET_PATCH (dev, ch, smp);
 }
 
 
@@ -160,17 +172,31 @@ static void setvol (int ch, int vol)
 
 
 static void setnote (int ch, int note)
+{
+    SEQ_START_NOTE (dev, ch, note, 0);
+}
+
+
+static void seteffect (int ch, int type, int val)
 {
 #ifdef AWE_DEVICE
-    /* AWE chorus and reverb handling by Takashi Iwai */
     if (si.synth_subtype == SAMPLE_TYPE_AWE32) {
-	SEQ_STOP_NOTE (dev, ch, note, 0);
-	/* chorus & reverb level: 0 - 255 */
-	AWE_SEND_EFFECT (dev, ch, AWE_FX_CHORUS, choruslevel);
-	AWE_SEND_EFFECT (dev, ch, AWE_FX_REVERB, reverblevel);
+	switch (type) {
+	case XMP_FX_CHORUS:
+	    AWE_SEND_EFFECT (dev, ch, AWE_FX_CHORUS, val);
+	    break;
+	case XMP_FX_REVERB:
+	    AWE_SEND_EFFECT (dev, ch, AWE_FX_REVERB, val);
+	    break;
+	case XMP_FX_CUTOFF:
+	    AWE_SEND_EFFECT (dev, ch, AWE_FX_CUTOFF, val);
+	    break;
+	case XMP_FX_RESONANCE:
+	    AWE_SEND_EFFECT (dev, ch, AWE_FX_FILTERQ, val / 16);
+	    break;
+	}
     }
 #endif
-    SEQ_START_NOTE (dev, ch, note, 0);
 }
 
 
@@ -197,6 +223,7 @@ static void starttimer ()
 static void stoptimer ()
 {
     SEQ_STOP_TIMER ();
+    bufdump ();
 }
 
 
@@ -206,21 +233,17 @@ static void resetvoices ()
 
 #ifdef AWE_DEVICE
     if (si.synth_subtype == SAMPLE_TYPE_AWE32) {
-	if (chorusmode) {
-	    AWE_CHORUS_MODE (dev, chorusmode);
-	}
-	if (reverbmode) {
-	    AWE_REVERB_MODE (dev, reverbmode);
-	}
+	AWE_CHORUS_MODE (dev, chorusmode);
+	AWE_REVERB_MODE (dev, reverbmode);
     }
 #endif
-    for (i = 0; i < 32; i++) {
+    for (i = 0; i < SEQ_NUM_VOICES; i++) {
+	SEQ_STOP_NOTE (dev, i, 255, 0);
 	SEQ_EXPRESSION (dev, i, 255);
 	SEQ_MAIN_VOLUME (dev, i, 100);
 	SEQ_CONTROL (dev, i, CTRL_PITCH_BENDER_RANGE, 8191);
 	SEQ_BENDER (dev, i, 0);
 	SEQ_PANNING (dev, i, 0);
-	SEQ_STOP_NOTE (dev, i, 255, 0);
 	bufdump ();
     }
 }
@@ -248,12 +271,12 @@ static void bufdump ()
 	select (seqfd + 1, &rfds, &wfds, NULL, NULL);
 
 	if (FD_ISSET (seqfd, &rfds)) {
-	    if ((read (seqfd, &__echo_msg, 4) == 4) &&
-		((__echo_msg & 0xff) == SEQ_ECHO)) {
-		__echo_msg >>= 8;
-		__event_callback (__echo_msg);
+	    if ((read (seqfd, &echo_msg, 4) == 4) &&
+		((echo_msg & 0xff) == SEQ_ECHO)) {
+		echo_msg >>= 8;
+		xmp_event_callback (echo_msg);
 	    } else
-		__echo_msg = 0;		/* ECHO_NONE */
+		echo_msg = 0;		/* ECHO_NONE */
 	}
 
 	if (FD_ISSET (seqfd, &wfds) && ((j = _seqbufptr) != 0)) {
@@ -285,48 +308,56 @@ static void seq_sync (double next_time)
     if (next_time == 0)
 	this_time = 0;
 
+    if (next_time - this_time < 1.0)
+	return;
+
     if (next_time > this_time) {
+#ifdef HZ
+	/* Nils Faerber's hack */
+	SEQ_WAIT_TIME (next_time * HZ / 100);
+#else
 	SEQ_WAIT_TIME (next_time);
+#endif
 	this_time = next_time;
     }
 }
 
 
-static void writepatch (struct patch_info *patch)
+static int writepatch (struct patch_info *patch)
 {
     struct sbi_instrument sbi;
 
     if (!patch) {
 	clearmem ();
-	return;
+	return XMP_OK;
     }
 
-    if (__mode_fm ^ (patch->len == -1))
-	return;
+    if ((!!(xmp_ctl->outfmt & XMP_FMT_FM)) ^ (patch->len == XMP_PATCH_FM))
+	return XMP_ERR_TPTCH;
 
-    if (patch->len == -1) {
+    patch->device_no = dev;
+    if (patch->len == XMP_PATCH_FM) {
 	sbi.key = FM_PATCH;
 	sbi.device = dev;
 	sbi.channel = patch->instr_no;
 	memcpy (&sbi.operators, patch->data, 11);
 	write (seqfd, &sbi, sizeof (sbi));
+
+	return XMP_OK;
     }
+    SEQ_WRPATCH (patch, sizeof (struct patch_info) + patch->len - 1);
 
-    patch->device_no = dev;
-    SEQ_WRPATCH (patch, sizeof (*patch) + patch->len - 1);
-
-    if (patch->len != -1)
-	free (patch);
+    return XMP_OK;
 }
 
 
 static int getmsg ()
 {
-    return __echo_msg;
+    return echo_msg;
 }
 
 
-static int init (struct drv_config *cfg)
+static int init (struct xmp_control *ctl)
 {
     int found;
     char *buf, *token;
@@ -335,21 +366,21 @@ static int init (struct drv_config *cfg)
     buf = calloc (1, 256);
 
     parm_init ();
-    chkparm2 ("chorus", "%d,%d", &chorusmode, &choruslevel);
-    chkparm2 ("reverb", "%d,%d", &reverbmode, &reverblevel);
-    chkparm1 ("opl2", __mode_fm = 1);
+    chkparm1 ("awechorus", chorusmode = strtoul (token, NULL, 0));
+    chkparm1 ("awereverb", reverbmode = strtoul (token, NULL, 0));
+    chkparm1 ("opl2", ctl->outfmt |= XMP_FMT_FM);
     chkparm1 ("dev", dev_sequencer = token);
     parm_end ();
 
     if ((seqfd = open (dev_sequencer, O_RDWR)) != -1) {
 	if ((seqfd != -1) && (ioctl (seqfd, SNDCTL_SEQ_NRSYNTHS, &dev) == -1)) {
 	    fprintf (stderr, "xmp: can't determine number of synths\n");
-	    return 0;
+	    return XMP_ERR_DINIT;
 	}
     } else {
-	if (opt.verbose > 2)
+	if (ctl->verbose > 2)
 	    fprintf (stderr, "xmp: can't open sequencer\n");
-	return 0;
+	return XMP_ERR_DINIT;
     }
 
     for (found = 0; dev--; ) {
@@ -357,29 +388,35 @@ static int init (struct drv_config *cfg)
 
 	if (ioctl (seqfd, SNDCTL_SYNTH_INFO, &si) == -1) {
 	    fprintf (stderr, "xmp: can't determine synth info\n");
-	    return 0;
+	    return XMP_ERR_SYNTH;
 	}
 
-	if (si.synth_type == (__mode_fm ? SYNTH_TYPE_FM : SYNTH_TYPE_SAMPLE)) {
-	    if (!__mode_fm) {
+	if (si.synth_type == (ctl->outfmt & XMP_FMT_FM ?
+	    SYNTH_TYPE_FM : SYNTH_TYPE_SAMPLE)) {
+	    if (si.synth_type != SYNTH_TYPE_FM) {
 		int i = dev;
 		ioctl(seqfd, SNDCTL_SEQ_RESETSAMPLES, &i);
 		i = dev;
 		ioctl(seqfd, SNDCTL_SYNTH_MEMAVL, &i);
-		if (!i) {
+		if (!i)
 		    continue;
-		}
+		ctl->memavl = i;
 	    }
 
 	    sprintf (buf, "%s [%s]", drv_oss_seq.description, si.name);
-	    if (si.synth_subtype == 0x20) {
+#ifdef AWE_DEVICE
+	    if (si.synth_subtype == SAMPLE_TYPE_AWE32) {
 		strcat (buf,
-		    "\nAWE support Copyright (C) 1996,1997 by Takashi Iwai");
+		    "\nAWE support Copyright (C) 1996,1997 Takashi Iwai");
+
+#ifdef HAVE_AWE_MD_NEW_VOLUME_CALC
+		AWE_MISC_MODE (dev, AWE_MD_NEW_VOLUME_CALC, 0);
+#endif
 	    }
-	
+
+#endif
 	    drv_oss_seq.description = buf;
 
-	    cfg->rate = 0;
 	    found = 1;
 	    break;
 	}
@@ -387,14 +424,14 @@ static int init (struct drv_config *cfg)
 
     if (!found) {
 	close (seqfd);
-	return 0;
+	return XMP_ERR_DINIT;
     }
 
     SEQ_VOLUME_MODE (dev, VOL_METHOD_LINEAR);
     bufdump ();
     ioctl (seqfd, SNDCTL_SEQ_SYNC, 0);
 
-    return 1;
+    return XMP_OK;
 }
 
 
