@@ -1,8 +1,8 @@
-/* Extended Module Player - period.h
- * Copyright (C) 1996,1997 Claudio Matsuoka and Hipolito Carraro Jr
+/* Extended Module Player
+ * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See docs/COPYING
+ * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
  */
 
@@ -27,6 +27,6 @@ int	note_to_period		(int, int, int);
 int	note_to_period2		(int, int);
 int	period_to_note		(int);
 int	period_to_bend		(int, int, int, int, int, int);
-void	c2spd_to_note		(int, char *, char *);
+void	c2spd_to_note		(int, int *, int *);
 
 #endif /* __PERIOD_H */
