@@ -1,8 +1,8 @@
-/* Extended Module Player - mtm.h
- * Copyright (C) 1996,1997 Claudio Matsuoka and Hipolito Carraro Jr
+/* Extended Module Player
+ * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See docs/COPYING
+ * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
  */
 
@@ -24,7 +24,7 @@ struct mtm_file_header {
 struct mtm_instrument_header {
     uint8 name[22];		/* Instrument name */
     uint32 length;		/* Instrument length in bytes */
-    uint32 loopstart;		/* Sample loop start */
+    uint32 loop_start;		/* Sample loop start */
     uint32 loopend;		/* Sample loop end */
     uint8 finetune;		/* Finetune */
     uint8 volume;		/* Playback volume */
