@@ -1,8 +1,8 @@
-/* Extended Module Player - xpanel.h
+/* Extended Module Player
  * Copyright (C) 1996, 1997 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See docs/COPYING
+ * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
  */
 
@@ -19,6 +19,7 @@ struct font_header {
 
 struct ipc_info {
     struct xmp_module_info mi;
+    char filename[80];
     int vol[32];
     int mute[32];
     int progress;
@@ -30,6 +31,7 @@ struct ipc_info {
 };
 
 int	setcolor	(int);
+void    settitle        (char *);
 void	putimage	(int, int, int, int);
 void	update_display	(void);
 void	prepare_screen	(void);
