@@ -1,8 +1,8 @@
-/* Extended Module Player - player.h
- * Copyright (C) 1996,1997 Claudio Matsuoka and Hipolito Carraro Jr
+/* Extended Module Player
+ * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See docs/COPYING
+ * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
  */
 
@@ -13,36 +13,12 @@
 #define RESET(f) RESET_FLAG (xc->flags,f)
 #define TEST(f)	TEST_FLAG (xc->flags,f)
 
-/* t = envelope flags		i = index
- * e = envelope points		s = sustain point
- * a = loop start point		b = loop end point
- */
-#define DO_ENVELOPE(t,i,e,s,a,b) { if ((!(t & XXM_ENV_SUS) || xc->release \
-    || (i < e[s * 2])) && (i < 0xffff)) i++; if ((t & XXM_ENV_LOOP) \
-    && (i >= e[b * 2])) i = e[a * 2]; }
-
-/* Pattern loop stack size */
-#define ROW_MAX		0x0100
-
 /* Global flags */
 #define PATTERN_BREAK	0x0001 
 #define PATTERN_LOOP	0x0002 
 #define MODULE_ABORT	0x0004 
 #define MODULE_RESTART	0x0008 
 
-#if 0
-#define CLEAR_CHANNEL(x) { \
-    memset ((uint8 *) &xc[x] + 2 * sizeof (int) + sizeof (void *), 0, \
-    sizeof (xc[x]) - 2 * sizeof (int) - sizeof (void *)); \
-    xc[x].ins = -1; }
-#endif
-
-/* ????? */
-
-#define CLEAR_CHANNEL(x) { \
-    memset (&xc[x], 0, sizeof (xc[x])); \
-    xc[x].ins = -1; xc[x].masterpan = xxc[x].pan; }
-
 #define XXIH xxih[xc->ins]
 #define XXIM xxim[xc->ins]
 #define XXAE xxae[xc->ins]
@@ -50,27 +26,51 @@
 #define XXFE xxfe[xc->ins]
 #define XXI xxi[xc->ins]
 
+#define DOENV_RELEASE	((TEST (RELEASE) || act == XMP_ACT_OFF))
+
 struct retrig_t {
     int s;
     int m;
     int d;
 };
 
+struct flow_control {
+    int pbreak;
+    int jump;
+    int delay;
+    int jumpline;
+    int row_cnt;
+    int loop_chn;
+    int* loop_row;
+    int* loop_stack;
+};
+
 /* The following macros are used to set the flags for each channel */
-#define VOL_SLIDE	0x0001
-#define PAN_SLIDE	0x0002
-#define TONEPORTA	0x0004
-#define PITCHBEND	0x0008
-#define VIBRATO		0x0010
-#define TREMOLO		0x0020
-#define FINE_VOLS	0x0040
-#define FINE_BEND	0x0080
-/* These don't need to be "persistent" between frames */
-#define NEW_NOTE	0x0100
-#define NEW_VOL		0x0200
-#define NEW_PAN		0x0400
-#define NEW_INS		0x0800
-#define FINETUNE	0x1000
+#define VOL_SLIDE	0x00001
+#define PAN_SLIDE	0x00002
+#define TONEPORTA	0x00004
+#define PITCHBEND	0x00008
+#define VIBRATO		0x00010
+#define TREMOLO		0x00020
+#define FINE_VOLS	0x00040
+#define FINE_BEND	0x00080
+#define NEW_PAN		0x00100
+#define FINETUNE	0x00200
+#define OFFSET		0x00400
+#define TRK_VSLIDE	0x00800
+#define TRK_FVSLIDE	0x01000
+#define RESET_VOL	0x02000
+#define RESET_ENV	0x04000
+#define IS_VALID	0x08000
+#define NEW_INS		0x10000
+#define NEW_VOL		0x20000
+#define ECHOBACK	0x40000
+#define VOL_SLIDE_2	0x80000
+
+/* These need to be "persistent" between frames */
+#define IS_READY	0x1000000
+#define FADEOUT		0x2000000
+#define RELEASE		0x4000000
 
 /* Prefixes: 
  * a_ for arpeggio
@@ -90,19 +90,34 @@ struct xmp_channel {
     int pitchbend;		/* Pitch bender value */
     int finetune;		/* Guess what */
     int ins;			/* Instrument number */
-    int oldins;			/* Previous instrument number */
+    int smp;			/* Sample number */
+    int insdef;			/* Instrument default */
     int pan;			/* Current pan */
     int masterpan;		/* Master pan -- for S3M set pan effect */
+    int mastervol;		/* Master vol -- for IT track vol effect */
     int delay;			/* Note delay in frames */
     int retrig;			/* Retrig delay in frames */
-    int rval;
+    int rval;			/* Retrig parameters */
     int rtype;			/* Retrig type */
     int rcount;			/* Retrig counter */
+    int tremor;			/* Tremor */
+    int tcnt_up;		/* Tremor counter (up cycle) */
+    int tcnt_dn;		/* Tremor counter (down cycle) */
+    int keyoff;			/* Key off counter */
     int volume;			/* Current volume */
+    int gvl;			/* Global volume for instrument for IT */
     int v_val;			/* Volume slide value */
     int v_fval;			/* Fine volume slide value */
+    int v_val2;			/* Volume slide value */
+    int volslide;		/* Volume slide effect memory */
+    int fvolslide;		/* Fine volume slide effect memory */
     int p_val;			/* Current pan value */
+    int trk_val;		/* Track volume slide value */
+    int trk_fval;		/* Track fine volume slide value */
+    int trkvsld;		/* Track volume slide effect memory */
     uint16 v_idx;		/* Volume envelope index */
+    uint16 p_idx;		/* Pan envelope index */
+    uint16 f_idx;		/* Freq envelope index */
     int y_type;			/* Vibrato waveform */
     int y_depth;		/* Vibrato depth */
     int y_sweep;		/* Vibrato sweep */
@@ -114,8 +129,7 @@ struct xmp_channel {
     int t_idx;			/* Tremolo index */
     int f_val;			/* Frequency slide value */
     int f_fval;			/* Fine frequency slide value */
-    uint16 p_idx;		/* Pan envelope index */
-    int release;		/* Note released */
+    int porta;			/* Portamento effect memory */
     int fadeout;		/* Current fadeout (release) value */
     int gliss;			/* Glissando active */
     int s_end;			/* Target period for tone portamento */
@@ -126,8 +140,22 @@ struct xmp_channel {
     int insvib_idx;		/* Instrument vibrato index */
     int insvib_swp;		/* Instrument vibrato sweep */
     int offset;			/* Sample offset */
+    int cutoff;			/* IT filter cutoff frequency */
+    int resonance;		/* IT filter resonance */
+    int med_vp;			/* MED synth volume sequence table pointer */
+    int med_vv;			/* MED synth volume slide value */
+    int med_vs;			/* MED synth volume speed */
+    int med_vc;			/* MED synth volume speed counter */
+    int med_vw;			/* MED synth volume wait counter */
+    int med_wp;			/* MED synth waveform sequence table pointer */
+    int med_wv;			/* MED synth waveform slide value */
+    int med_ws;			/* MED synth waveform speed */
+    int med_wc;			/* MED synth waveform speed counter */
+    int med_ww;			/* MED synth waveform wait counter */
+    int med_period;		/* MED synth period for RES */
 };
 
-void process_fx (uint8, uint8, uint8, struct xmp_channel *);
+void process_fx (int, uint8, uint8, uint8, struct xmp_channel *);
+void xmp_med_synth (int, struct xmp_channel *, int);
 
 #endif /* __PLAYER_H */
